@@ -3,7 +3,10 @@ from typing import List, Literal, Tuple
 import torch
 import torchvision
 from inference_exp.entities import ImageDimensions
-from inference_exp.models.common.roboflow.model_packages import PreProcessingMetadata
+from inference_exp.models.common.roboflow.model_packages import (
+    PreProcessingMetadata,
+    StaticCropOffset,
+)
 from torchvision.transforms import functional
 
 
@@ -57,6 +60,21 @@ def run_nms_for_object_detection(
     return results
 
 
+def post_process_nms_fused_model_output(
+    output: torch.Tensor,
+    conf_thresh: float = 0.25,
+) -> List[torch.Tensor]:
+    bs = output.shape[0]
+    nms_results = []
+    for batch_element_id in range(bs):
+        batch_element_result = output[batch_element_id]
+        batch_element_result = batch_element_result[
+            batch_element_result[:, 4] >= conf_thresh
+        ]
+        nms_results.append(batch_element_result)
+    return nms_results
+
+
 def run_nms_for_instance_segmentation(
     output: torch.Tensor,
     conf_thresh: float = 0.25,
@@ -198,6 +216,21 @@ def rescale_image_detections(
         device=image_detections.device,
     )
     image_detections[:, :4].div_(scale)
+    if (
+        image_metadata.static_crop_offset.offset_x != 0
+        or image_metadata.static_crop_offset.offset_y != 0
+    ):
+        static_crop_offsets = torch.as_tensor(
+            [
+                image_metadata.static_crop_offset.offset_x,
+                image_metadata.static_crop_offset.offset_y,
+                image_metadata.static_crop_offset.offset_x,
+                image_metadata.static_crop_offset.offset_y,
+            ],
+            dtype=image_detections.dtype,
+            device=image_detections.device,
+        )
+        image_detections[:, :4].add_(static_crop_offsets)
     return image_detections
 
 
@@ -230,13 +263,40 @@ def rescale_key_points_detections(
             dtype=image_detections.dtype,
             device=image_detections.device,
         ).repeat(key_points_slots_in_prediction)
-        image_detections[:, 5 + num_classes :].sub_(key_points_offsets)
+        image_detections[:, 6:].sub_(key_points_offsets)
         key_points_scale = torch.as_tensor(
             [metadata.scale_width, metadata.scale_height, 1.0],
             dtype=image_detections.dtype,
             device=image_detections.device,
         ).repeat(key_points_slots_in_prediction)
-        image_detections[:, 5 + num_classes :].div_(key_points_scale)
+        image_detections[:, 6:].div_(key_points_scale)
+        if (
+            metadata.static_crop_offset.offset_x != 0
+            or metadata.static_crop_offset.offset_y != 0
+        ):
+            static_crop_offset_length = (image_detections.shape[1] - 6) // 3
+            static_crop_offsets = torch.as_tensor(
+                [
+                    metadata.static_crop_offset.offset_x,
+                    metadata.static_crop_offset.offset_y,
+                    0,
+                ]
+                * static_crop_offset_length,
+                dtype=image_detections.dtype,
+                device=image_detections.device,
+            )
+            image_detections[:, 6:].add_(static_crop_offsets)
+            static_crop_offsets = torch.as_tensor(
+                [
+                    metadata.static_crop_offset.offset_x,
+                    metadata.static_crop_offset.offset_y,
+                    metadata.static_crop_offset.offset_x,
+                    metadata.static_crop_offset.offset_y,
+                ],
+                dtype=image_detections.dtype,
+                device=image_detections.device,
+            )
+            image_detections[:, :4].add_(static_crop_offsets)
     return detections
 
 
@@ -273,11 +333,14 @@ def align_instance_segmentation_results(
     scale_width: float,
     scale_height: float,
     original_size: ImageDimensions,
+    size_after_pre_processing: ImageDimensions,
     inference_size: ImageDimensions,
+    static_crop_offset: StaticCropOffset,
+    binarization_threshold: float = 0.0,
 ) -> Tuple[torch.Tensor, torch.Tensor]:
     if image_bboxes.shape[0] == 0:
         empty_masks = torch.empty(
-            size=(0, original_size.height, original_size.width),
+            size=(0, size_after_pre_processing.height, size_after_pre_processing.width),
             dtype=torch.bool,
             device=image_bboxes.device,
         )
@@ -303,16 +366,70 @@ def align_instance_segmentation_results(
         round(mask_w_scale * pad_left),
         round(mask_w_scale * pad_right),
     )
-    masks = masks[
-        :, mask_pad_top : mh - mask_pad_bottom, mask_pad_left : mw - mask_pad_right
-    ]
+    if (
+        mask_pad_top < 0
+        or mask_pad_bottom < 0
+        or mask_pad_left < 0
+        or mask_pad_right < 0
+    ):
+        masks = torch.nn.functional.pad(
+            masks,
+            (
+                abs(min(mask_pad_left, 0)),
+                abs(min(mask_pad_right, 0)),
+                abs(min(mask_pad_top, 0)),
+                abs(min(mask_pad_bottom, 0)),
+            ),
+            "constant",
+            0,
+        )
+        padded_mask_offset_top = max(mask_pad_top, 0)
+        padded_mask_offset_bottom = max(mask_pad_bottom, 0)
+        padded_mask_offset_left = max(mask_pad_left, 0)
+        padded_mask_offset_right = max(mask_pad_right, 0)
+        masks = masks[
+            :,
+            padded_mask_offset_top : masks.shape[1] - padded_mask_offset_bottom,
+            padded_mask_offset_left : masks.shape[2] - padded_mask_offset_right,
+        ]
+    else:
+        masks = masks[
+            :, mask_pad_top : mh - mask_pad_bottom, mask_pad_left : mw - mask_pad_right
+        ]
     masks = (
         functional.resize(
             masks,
-            [original_size.height, original_size.width],
+            [size_after_pre_processing.height, size_after_pre_processing.width],
             interpolation=functional.InterpolationMode.BILINEAR,
         )
-        .gt_(0.0)
+        .gt_(binarization_threshold)
         .to(dtype=torch.bool)
     )
+    if static_crop_offset.offset_x > 0 or static_crop_offset.offset_y > 0:
+        mask_canvas = torch.zeros(
+            (
+                masks.shape[0],
+                original_size.height,
+                original_size.width,
+            ),
+            dtype=torch.bool,
+            device=masks.device,
+        )
+        mask_canvas[
+            :,
+            static_crop_offset.offset_y : static_crop_offset.offset_y + masks.shape[1],
+            static_crop_offset.offset_x : static_crop_offset.offset_x + masks.shape[2],
+        ] = masks
+        static_crop_offsets = torch.as_tensor(
+            [
+                static_crop_offset.offset_x,
+                static_crop_offset.offset_y,
+                static_crop_offset.offset_x,
+                static_crop_offset.offset_y,
+            ],
+            dtype=image_bboxes.dtype,
+            device=image_bboxes.device,
+        )
+        image_bboxes[:, :4].add_(static_crop_offsets)
+        masks = mask_canvas
     return image_bboxes, masks
