@@ -60,6 +60,22 @@ class InferenceTRTLogger(trt.ILogger):
 TRT_LOGGER = InferenceTRTLogger()
 
 
+def get_engine_inputs_and_outputs(
+    engine: trt.ICudaEngine,
+) -> Tuple[List[str], List[str]]:
+    num_inputs = engine.num_io_tensors
+    inputs = []
+    outputs = []
+    for i in range(num_inputs):
+        name = engine.get_tensor_name(i)
+        io_mode = engine.get_tensor_mode(name)
+        if io_mode == trt.TensorIOMode.INPUT:
+            inputs.append(name)
+        elif io_mode == trt.TensorIOMode.OUTPUT:
+            outputs.append(name)
+    return inputs, outputs
+
+
 def infer_from_trt_engine(
     pre_processed_images: torch.Tensor,
     trt_config: TRTConfig,
@@ -180,8 +196,18 @@ def execute_trt_engine(
         )
         context.set_tensor_address(output, result.data_ptr())
         results.append(result)
-    context.set_input_shape(input_name, tuple(pre_processed_images.shape))
-    context.set_tensor_address(input_name, pre_processed_images.data_ptr())
+    status = context.set_input_shape(input_name, tuple(pre_processed_images.shape))
+    if not status:
+        raise ModelRuntimeError(
+            message="Failed to set TRT model input shape during forward pass from the model.",
+            help_url="https://todo",
+        )
+    status = context.set_tensor_address(input_name, pre_processed_images.data_ptr())
+    if not status:
+        raise ModelRuntimeError(
+            message="Failed to set input tensor data pointer during forward pass from the model.",
+            help_url="https://todo",
+        )
     stream = torch.cuda.Stream(device=device)
     status = context.execute_async_v3(stream_handle=stream.cuda_stream)
     if not status:
@@ -237,12 +263,3 @@ def load_model(
             "https://github.com/roboflow/inference/issues",
             help_url="https://todo",
         ) from error
-
-
-def get_output_tensor_names(engine: trt.ICudaEngine) -> List[str]:
-    output_names = []
-    for i in range(engine.num_io_tensors):
-        name = engine.get_tensor_name(i)
-        if engine.get_tensor_mode(name) == trt.TensorIOMode.OUTPUT:
-            output_names.append(name)
-    return output_names
