@@ -1,12 +1,24 @@
-from typing import List, Union, Optional
 import os
+from typing import List, Optional, Tuple, Union
 
 import numpy as np
 import torch
-from peft import PeftModel
 from inference_exp.configuration import DEFAULT_DEVICE
 from inference_exp.entities import ColorFormat
-from transformers import AutoProcessor, PaliGemmaForConditionalGeneration
+from inference_exp.models.common.roboflow.model_packages import (
+    InferenceConfig,
+    ResizeMode,
+    parse_inference_config,
+)
+from inference_exp.models.common.roboflow.pre_processing import (
+    pre_process_network_input,
+)
+from peft import PeftModel
+from transformers import (
+    AutoProcessor,
+    BitsAndBytesConfig,
+    PaliGemmaForConditionalGeneration,
+)
 
 
 class PaliGemmaHF:
@@ -16,50 +28,92 @@ class PaliGemmaHF:
         cls,
         model_name_or_path: str,
         device: torch.device = DEFAULT_DEVICE,
+        trust_remote_code: bool = False,
+        local_files_only: bool = True,
+        quantization_config: Optional[BitsAndBytesConfig] = None,
+        disable_quantization: bool = False,
         **kwargs,
     ) -> "PaliGemmaHF":
         torch_dtype = torch.float16 if device.type == "cuda" else torch.float32
-
+        inference_config_path = os.path.join(
+            model_name_or_path, "inference_config.json"
+        )
+        inference_config = None
+        if os.path.exists(inference_config_path):
+            inference_config = parse_inference_config(
+                config_path=inference_config_path,
+                allowed_resize_modes={
+                    ResizeMode.STRETCH_TO,
+                    ResizeMode.LETTERBOX,
+                    ResizeMode.CENTER_CROP,
+                    ResizeMode.LETTERBOX_REFLECT_EDGES,
+                },
+            )
+        if (
+            quantization_config is None
+            and device.type == "cuda"
+            and not disable_quantization
+        ):
+            quantization_config = BitsAndBytesConfig(
+                load_in_4bit=True,
+                bnb_4bit_quant_type="nf4",
+                bnb_4bit_compute_dtype=torch.bfloat16,
+            )
         adapter_config_path = os.path.join(model_name_or_path, "adapter_config.json")
         if os.path.exists(adapter_config_path):
             base_model_path = os.path.join(model_name_or_path, "base")
             model = PaliGemmaForConditionalGeneration.from_pretrained(
                 base_model_path,
-                torch_dtype=torch_dtype,
-                trust_remote_code=True,
-                local_files_only=True,
+                dtype=torch_dtype,
+                trust_remote_code=trust_remote_code,
+                local_files_only=local_files_only,
+                quantization_config=quantization_config,
             )
             model = PeftModel.from_pretrained(model, model_name_or_path)
-            model.merge_and_unload()
+            if quantization_config is None:
+                model.merge_and_unload()
             model.to(device)
 
             processor = AutoProcessor.from_pretrained(
-                base_model_path, trust_remote_code=True, local_files_only=True
+                base_model_path,
+                trust_remote_code=trust_remote_code,
+                local_files_only=local_files_only,
+                use_fast=True,
             )
         else:
             model = PaliGemmaForConditionalGeneration.from_pretrained(
                 model_name_or_path,
-                torch_dtype=torch_dtype,
+                dtype=torch_dtype,
                 device_map=device,
-                trust_remote_code=True,
-                local_files_only=True,
+                trust_remote_code=trust_remote_code,
+                local_files_only=local_files_only,
+                quantization_config=quantization_config,
             ).eval()
             processor = AutoProcessor.from_pretrained(
-                model_name_or_path, trust_remote_code=True, local_files_only=True
+                model_name_or_path,
+                trust_remote_code=trust_remote_code,
+                local_files_only=local_files_only,
+                use_fast=True,
             )
         return cls(
-            model=model, processor=processor, device=device, torch_dtype=torch_dtype
+            model=model,
+            processor=processor,
+            inference_config=inference_config,
+            device=device,
+            torch_dtype=torch_dtype,
         )
 
     def __init__(
         self,
         model: PaliGemmaForConditionalGeneration,
         processor: AutoProcessor,
+        inference_config: Optional[InferenceConfig],
         device: torch.device,
         torch_dtype: torch.dtype,
     ):
         self._model = model
         self._processor = processor
+        self._inference_config = inference_config
         self._device = device
         self._torch_dtype = torch_dtype
 
@@ -91,6 +145,7 @@ class PaliGemmaHF:
         images: Union[torch.Tensor, List[torch.Tensor], np.ndarray, List[np.ndarray]],
         prompt: str,
         input_color_format: Optional[ColorFormat] = None,
+        image_size: Optional[Tuple[int, int]] = None,
         **kwargs,
     ) -> dict:
         def _to_tensor(image: Union[np.ndarray, torch.Tensor]) -> torch.Tensor:
@@ -103,13 +158,23 @@ class PaliGemmaHF:
                 tensor_image = tensor_image[[2, 1, 0], :, :]
             return tensor_image
 
-        if isinstance(images, torch.Tensor) and images.ndim > 3:
-            image_list = [_to_tensor(img) for img in images]
-        elif not isinstance(images, list):
-            image_list = [_to_tensor(images)]
+        if self._inference_config is None:
+            if isinstance(images, torch.Tensor) and images.ndim > 3:
+                image_list = [_to_tensor(img) for img in images]
+            elif not isinstance(images, list):
+                image_list = [_to_tensor(images)]
+            else:
+                image_list = [_to_tensor(img) for img in images]
         else:
-            image_list = [_to_tensor(img) for img in images]
-
+            images = pre_process_network_input(
+                images=images,
+                image_pre_processing=self._inference_config.image_pre_processing,
+                network_input=self._inference_config.network_input,
+                target_device=self._device,
+                input_color_format=input_color_format,
+                image_size_wh=image_size,
+            )[0]
+            image_list = [e[0] for e in torch.split(images, 1, dim=0)]
         num_images = len(image_list)
 
         if isinstance(prompt, str) and num_images > 1:
