@@ -372,15 +372,14 @@ class WindowedDinov2WithRegistersEmbeddings(nn.Module):
             num_w_patches_per_window = num_w_patches // self.config.num_windows
             num_h_patches_per_window = num_h_patches // self.config.num_windows
             num_windows = self.config.num_windows
-            windowed_pixel_tokens = pixel_tokens_with_pos_embed.view(
-                batch_size,
-                num_windows,
+            windowed_pixel_tokens = pixel_tokens_with_pos_embed.reshape(
+                batch_size * num_windows,
                 num_h_patches_per_window,
                 num_windows,
                 num_h_patches_per_window,
                 -1,
             )
-            windowed_pixel_tokens = windowed_pixel_tokens.permute(0, 1, 3, 2, 4, 5)
+            windowed_pixel_tokens = windowed_pixel_tokens.permute(0, 2, 1, 3, 4)
             windowed_pixel_tokens = windowed_pixel_tokens.reshape(
                 batch_size * num_windows**2,
                 num_h_patches_per_window * num_w_patches_per_window,
@@ -1293,15 +1292,14 @@ class WindowedDinov2WithRegistersBackbone(
                         hidden_state = hidden_state.reshape(
                             B // num_windows_squared, num_windows_squared * HW, C
                         )
-                        hidden_state = hidden_state.view(
-                            B // num_windows_squared,
-                            self.config.num_windows,
+                        hidden_state = hidden_state.reshape(
+                            (B // num_windows_squared) * self.config.num_windows,
                             self.config.num_windows,
                             num_h_patches_per_window,
                             num_w_patches_per_window,
                             C,
                         )
-                        hidden_state = hidden_state.permute(0, 1, 3, 2, 4, 5)
+                        hidden_state = hidden_state.permute(0, 2, 1, 3, 4)
 
                     hidden_state = hidden_state.reshape(
                         batch_size, num_h_patches, num_w_patches, -1
