@@ -8,6 +8,7 @@ import torch.nn.functional as F
 import torchvision
 from inference_exp.models.rfdetr.backbone_builder import build_backbone
 from inference_exp.models.rfdetr.misc import NestedTensor
+from inference_exp.models.rfdetr.segmentation_head import SegmentationHead
 from inference_exp.models.rfdetr.transformer import build_transformer
 from pydantic import BaseModel, ConfigDict
 from torch import Tensor, nn
@@ -36,6 +37,10 @@ class ModelConfig(BaseModel):
     group_detr: int = 13
     gradient_checkpointing: bool = False
     positional_encoding_size: int
+    ia_bce_loss: bool = True
+    cls_loss_coef: float = 1.0
+    segmentation_head: bool = False
+    mask_downsample_ratio: int = 4
 
     model_config = ConfigDict(arbitrary_types_allowed=True)
 
@@ -102,6 +107,20 @@ class RFDETRMediumConfig(RFDETRBaseConfig):
     pretrain_weights: Optional[str] = "rf-detr-medium.pth"
 
 
+class RFDETRSegPreviewConfig(RFDETRBaseConfig):
+    segmentation_head: bool = True
+    out_feature_indexes: List[int] = [3, 6, 9, 12]
+    num_windows: int = 2
+    dec_layers: int = 4
+    patch_size: int = 12
+    resolution: int = 432
+    positional_encoding_size: int = 36
+    num_queries: int = 200
+    num_select: int = 200
+    pretrain_weights: Optional[str] = "rf-detr-seg-preview.pt"
+    num_classes: int = 90
+
+
 class LWDETR(nn.Module):
     """This is the Group DETR v3 module that performs object detection"""
 
@@ -109,6 +128,7 @@ class LWDETR(nn.Module):
         self,
         backbone,
         transformer,
+        segmentation_head,
         num_classes,
         num_queries,
         aux_loss=False,
@@ -134,7 +154,7 @@ class LWDETR(nn.Module):
         hidden_dim = transformer.d_model
         self.class_embed = nn.Linear(hidden_dim, num_classes)
         self.bbox_embed = MLP(hidden_dim, hidden_dim, 4, 3)
-
+        self.segmentation_head = segmentation_head
         query_dim = 4
         self.refpoint_embed = nn.Embedding(num_queries * group_detr, query_dim)
         self.query_feat = nn.Embedding(num_queries * group_detr, hidden_dim)
@@ -175,23 +195,29 @@ class LWDETR(nn.Module):
         self._export = False
 
     def reinitialize_detection_head(self, num_classes):
-        # Create new classification head
-        del self.class_embed
-        self.add_module("class_embed", nn.Linear(self.transformer.d_model, num_classes))
-
-        # Initialize with focal loss bias adjustment
-        prior_prob = 0.01
-        bias_value = -math.log((1 - prior_prob) / prior_prob)
-        self.class_embed.bias.data = torch.ones(num_classes) * bias_value
+        base = self.class_embed.weight.shape[0]
+        num_repeats = int(math.ceil(num_classes / base))
+        self.class_embed.weight.data = self.class_embed.weight.data.repeat(
+            num_repeats, 1
+        )
+        self.class_embed.weight.data = self.class_embed.weight.data[:num_classes]
+        self.class_embed.bias.data = self.class_embed.bias.data.repeat(num_repeats)
+        self.class_embed.bias.data = self.class_embed.bias.data[:num_classes]
 
         if self.two_stage:
-            del self.transformer.enc_out_class_embed
-            self.transformer.add_module(
-                "enc_out_class_embed",
-                nn.ModuleList(
-                    [copy.deepcopy(self.class_embed) for _ in range(self.group_detr)]
-                ),
-            )
+            for enc_out_class_embed in self.transformer.enc_out_class_embed:
+                enc_out_class_embed.weight.data = (
+                    enc_out_class_embed.weight.data.repeat(num_repeats, 1)
+                )
+                enc_out_class_embed.weight.data = enc_out_class_embed.weight.data[
+                    :num_classes
+                ]
+                enc_out_class_embed.bias.data = enc_out_class_embed.bias.data.repeat(
+                    num_repeats
+                )
+                enc_out_class_embed.bias.data = enc_out_class_embed.bias.data[
+                    :num_classes
+                ]
 
     def export(self):
         self._export = True
@@ -263,9 +289,20 @@ class LWDETR(nn.Module):
 
             outputs_class = self.class_embed(hs)
 
+            if self.segmentation_head is not None:
+                outputs_masks = self.segmentation_head(
+                    features[0].tensors, hs, samples.tensors.shape[-2:]
+                )
+
             out = {"pred_logits": outputs_class[-1], "pred_boxes": outputs_coord[-1]}
+            if self.segmentation_head is not None:
+                out["pred_masks"] = outputs_masks[-1]
             if self.aux_loss:
-                out["aux_outputs"] = self._set_aux_loss(outputs_class, outputs_coord)
+                out["aux_outputs"] = self._set_aux_loss(
+                    outputs_class,
+                    outputs_coord,
+                    outputs_masks if self.segmentation_head is not None else None,
+                )
 
         if self.two_stage:
             group_detr = self.group_detr if self.training else 1
@@ -276,11 +313,28 @@ class LWDETR(nn.Module):
                     hs_enc_list[g_idx]
                 )
                 cls_enc.append(cls_enc_gidx)
+
             cls_enc = torch.cat(cls_enc, dim=1)
+
+            if self.segmentation_head is not None:
+                masks_enc = self.segmentation_head(
+                    features[0].tensors,
+                    [
+                        hs_enc,
+                    ],
+                    samples.tensors.shape[-2:],
+                    skip_blocks=True,
+                )
+                masks_enc = torch.cat(masks_enc, dim=1)
+
             if hs is not None:
                 out["enc_outputs"] = {"pred_logits": cls_enc, "pred_boxes": ref_enc}
+                if self.segmentation_head is not None:
+                    out["enc_outputs"]["pred_masks"] = masks_enc
             else:
                 out = {"pred_logits": cls_enc, "pred_boxes": ref_enc}
+                if self.segmentation_head is not None:
+                    out["pred_masks"] = masks_enc
 
         return out
 
@@ -294,6 +348,8 @@ class LWDETR(nn.Module):
             srcs, None, poss, refpoint_embed_weight, query_feat_weight
         )
 
+        outputs_masks = None
+
         if hs is not None:
             if self.bbox_reparam:
                 outputs_coord_delta = self.bbox_embed(hs)
@@ -310,22 +366,50 @@ class LWDETR(nn.Module):
             else:
                 outputs_coord = (self.bbox_embed(hs) + ref_unsigmoid).sigmoid()
             outputs_class = self.class_embed(hs)
+            if self.segmentation_head is not None:
+                outputs_masks = self.segmentation_head(
+                    srcs[0],
+                    [
+                        hs,
+                    ],
+                    tensors.shape[-2:],
+                )[0]
         else:
             assert self.two_stage, "if not using decoder, two_stage must be True"
             outputs_class = self.transformer.enc_out_class_embed[0](hs_enc)
             outputs_coord = ref_enc
+            if self.segmentation_head is not None:
+                outputs_masks = self.segmentation_head(
+                    srcs[0],
+                    [
+                        hs_enc,
+                    ],
+                    tensors.shape[-2:],
+                    skip_blocks=True,
+                )[0]
 
-        return outputs_coord, outputs_class
+        if outputs_masks is not None:
+            return outputs_coord, outputs_class, outputs_masks
+        else:
+            return outputs_coord, outputs_class
 
     @torch.jit.unused
-    def _set_aux_loss(self, outputs_class, outputs_coord):
+    def _set_aux_loss(self, outputs_class, outputs_coord, outputs_masks):
         # this is a workaround to make torchscript happy, as torchscript
         # doesn't support dictionary with non-homogeneous values, such
         # as a dict having both a Tensor and a list.
-        return [
-            {"pred_logits": a, "pred_boxes": b}
-            for a, b in zip(outputs_class[:-1], outputs_coord[:-1])
-        ]
+        if outputs_masks is not None:
+            return [
+                {"pred_logits": a, "pred_boxes": b, "pred_masks": c}
+                for a, b, c in zip(
+                    outputs_class[:-1], outputs_coord[:-1], outputs_masks[:-1]
+                )
+            ]
+        else:
+            return [
+                {"pred_logits": a, "pred_boxes": b}
+                for a, b in zip(outputs_class[:-1], outputs_coord[:-1])
+            ]
 
     def update_drop_path(self, drop_path_rate, vit_encoder_num_layers):
         """ """
@@ -487,9 +571,19 @@ def build_model(config: ModelConfig) -> LWDETR:
         return backbone, None, None
     args.num_feature_levels = len(args.projector_scale)
     transformer = build_transformer(args)
+    segmentation_head = (
+        SegmentationHead(
+            args.hidden_dim,
+            args.dec_layers,
+            downsample_ratio=args.mask_downsample_ratio,
+        )
+        if args.segmentation_head
+        else None
+    )
     return LWDETR(
         backbone,
         transformer,
+        segmentation_head,
         num_classes=num_classes,
         num_queries=args.num_queries,
         aux_loss=args.aux_loss,
