@@ -6,15 +6,20 @@ import torch
 from inference_exp import Detections, ObjectDetectionModel
 from inference_exp.configuration import DEFAULT_DEVICE
 from inference_exp.entities import ColorFormat
-from inference_exp.errors import MissingDependencyError, ModelRuntimeError
+from inference_exp.errors import (
+    CorruptedModelPackageError,
+    MissingDependencyError,
+    ModelRuntimeError,
+)
 from inference_exp.models.common.cuda import use_cuda_context, use_primary_cuda_context
 from inference_exp.models.common.model_packages import get_model_package_contents
 from inference_exp.models.common.roboflow.model_packages import (
-    PreProcessingConfig,
+    InferenceConfig,
     PreProcessingMetadata,
+    ResizeMode,
     TRTConfig,
     parse_class_names_file,
-    parse_pre_processing_config,
+    parse_inference_config,
     parse_trt_config,
 )
 from inference_exp.models.common.roboflow.post_processing import (
@@ -23,7 +28,15 @@ from inference_exp.models.common.roboflow.post_processing import (
 from inference_exp.models.common.roboflow.pre_processing import (
     pre_process_network_input,
 )
-from inference_exp.models.common.trt import infer_from_trt_engine, load_model
+from inference_exp.models.common.trt import (
+    get_engine_inputs_and_outputs,
+    infer_from_trt_engine,
+    load_model,
+)
+from inference_exp.models.rfdetr.class_remapping import (
+    ClassesReMapping,
+    prepare_class_remapping,
+)
 
 try:
     import tensorrt as trt
@@ -72,7 +85,7 @@ class RFDetrForObjectDetectionTRT(
             model_package_dir=model_name_or_path,
             elements=[
                 "class_names.txt",
-                "environment.json",
+                "inference_config.json",
                 "trt_config.json",
                 "engine.plan",
             ],
@@ -80,9 +93,22 @@ class RFDetrForObjectDetectionTRT(
         class_names = parse_class_names_file(
             class_names_path=model_package_content["class_names.txt"]
         )
-        pre_processing_config = parse_pre_processing_config(
-            config_path=model_package_content["environment.json"],
+        inference_config = parse_inference_config(
+            config_path=model_package_content["inference_config.json"],
+            allowed_resize_modes={
+                ResizeMode.STRETCH_TO,
+                ResizeMode.LETTERBOX,
+                ResizeMode.CENTER_CROP,
+                ResizeMode.LETTERBOX_REFLECT_EDGES,
+            },
         )
+        classes_re_mapping = None
+        if inference_config.class_names_operations:
+            class_names, classes_re_mapping = prepare_class_remapping(
+                class_names=class_names,
+                class_names_operations=inference_config.class_names_operations,
+                device=device,
+            )
         trt_config = parse_trt_config(
             config_path=model_package_content["trt_config.json"]
         )
@@ -94,10 +120,29 @@ class RFDetrForObjectDetectionTRT(
                 engine_host_code_allowed=engine_host_code_allowed,
             )
             execution_context = engine.create_execution_context()
+        inputs, outputs = get_engine_inputs_and_outputs(engine=engine)
+        if len(inputs) != 1:
+            raise CorruptedModelPackageError(
+                message=f"Implementation assume single model input, found: {len(inputs)}.",
+                help_url="https://todo",
+            )
+        if len(outputs) != 2:
+            raise CorruptedModelPackageError(
+                message=f"Implementation assume 2 model outputs, found: {len(outputs)}.",
+                help_url="https://todo",
+            )
+        if "dets" not in outputs or "labels" not in outputs:
+            raise CorruptedModelPackageError(
+                message=f"Expected model outputs to be named `output0` and `output1`, but found: {outputs}.",
+                help_url="https://todo",
+            )
         return cls(
             engine=engine,
+            input_name=inputs[0],
+            output_names=["dets", "labels"],
             class_names=class_names,
-            pre_processing_config=pre_processing_config,
+            classes_re_mapping=classes_re_mapping,
+            inference_config=inference_config,
             trt_config=trt_config,
             device=device,
             cuda_context=cuda_context,
@@ -107,16 +152,22 @@ class RFDetrForObjectDetectionTRT(
     def __init__(
         self,
         engine: trt.ICudaEngine,
+        input_name: str,
+        output_names: List[str],
         class_names: List[str],
-        pre_processing_config: PreProcessingConfig,
+        classes_re_mapping: Optional[ClassesReMapping],
+        inference_config: InferenceConfig,
         trt_config: TRTConfig,
         device: torch.device,
         cuda_context: cuda.Context,
         execution_context: trt.IExecutionContext,
     ):
         self._engine = engine
-        self._pre_processing_config = pre_processing_config
+        self._input_name = input_name
+        self._output_names = output_names
+        self._inference_config = inference_config
         self._class_names = class_names
+        self._classes_re_mapping = classes_re_mapping
         self._device = device
         self._cuda_context = cuda_context
         self._execution_context = execution_context
@@ -135,11 +186,10 @@ class RFDetrForObjectDetectionTRT(
     ) -> Tuple[torch.Tensor, List[PreProcessingMetadata]]:
         return pre_process_network_input(
             images=images,
-            pre_processing_config=self._pre_processing_config,
-            expected_network_color_format="rgb",
+            image_pre_processing=self._inference_config.image_pre_processing,
+            network_input=self._inference_config.network_input,
             target_device=self._device,
             input_color_format=input_color_format,
-            normalization=([0.485, 0.456, 0.406], [0.229, 0.224, 0.225]),
         )
 
     def forward(
@@ -153,8 +203,8 @@ class RFDetrForObjectDetectionTRT(
                     engine=self._engine,
                     context=self._execution_context,
                     device=self._device,
-                    input_name="input",
-                    outputs=["dets", "labels"],
+                    input_name=self._input_name,
+                    outputs=self._output_names,
                 )
                 return detections, labels
 
@@ -179,6 +229,15 @@ class RFDetrForObjectDetectionTRT(
             confidence, sorted_indices = torch.sort(confidence, descending=True)
             top_classes = top_classes[sorted_indices]
             selected_boxes = selected_boxes[sorted_indices]
+            if self._classes_re_mapping is not None:
+                remapping_mask = torch.isin(
+                    top_classes, self._classes_re_mapping.remaining_class_ids
+                )
+                top_classes = self._classes_re_mapping.class_mapping[
+                    top_classes[remapping_mask]
+                ]
+                selected_boxes = selected_boxes[remapping_mask]
+                confidence = confidence[remapping_mask]
             cxcy = selected_boxes[:, :2]
             wh = selected_boxes[:, 2:]
             xy_min = cxcy - 0.5 * wh
