@@ -6,27 +6,37 @@ import torch
 from inference_exp import InstanceDetections, InstanceSegmentationModel
 from inference_exp.configuration import DEFAULT_DEVICE
 from inference_exp.entities import ColorFormat
-from inference_exp.errors import MissingDependencyError, ModelRuntimeError
+from inference_exp.errors import (
+    CorruptedModelPackageError,
+    MissingDependencyError,
+    ModelRuntimeError,
+)
 from inference_exp.models.common.cuda import use_cuda_context, use_primary_cuda_context
 from inference_exp.models.common.model_packages import get_model_package_contents
 from inference_exp.models.common.roboflow.model_packages import (
-    PreProcessingConfig,
+    InferenceConfig,
     PreProcessingMetadata,
+    ResizeMode,
     TRTConfig,
     parse_class_names_file,
-    parse_pre_processing_config,
+    parse_inference_config,
     parse_trt_config,
 )
 from inference_exp.models.common.roboflow.post_processing import (
     align_instance_segmentation_results,
     crop_masks_to_boxes,
+    post_process_nms_fused_model_output,
     preprocess_segmentation_masks,
     run_nms_for_instance_segmentation,
 )
 from inference_exp.models.common.roboflow.pre_processing import (
     pre_process_network_input,
 )
-from inference_exp.models.common.trt import infer_from_trt_engine, load_model
+from inference_exp.models.common.trt import (
+    get_engine_inputs_and_outputs,
+    infer_from_trt_engine,
+    load_model,
+)
 
 try:
     import tensorrt as trt
@@ -74,7 +84,7 @@ class YOLOv8ForInstanceSegmentationTRT(
             model_package_dir=model_name_or_path,
             elements=[
                 "class_names.txt",
-                "environment.json",
+                "inference_config.json",
                 "trt_config.json",
                 "engine.plan",
             ],
@@ -82,9 +92,20 @@ class YOLOv8ForInstanceSegmentationTRT(
         class_names = parse_class_names_file(
             class_names_path=model_package_content["class_names.txt"]
         )
-        pre_processing_config = parse_pre_processing_config(
-            config_path=model_package_content["environment.json"],
+        inference_config = parse_inference_config(
+            config_path=model_package_content["inference_config.json"],
+            allowed_resize_modes={
+                ResizeMode.STRETCH_TO,
+                ResizeMode.LETTERBOX,
+                ResizeMode.CENTER_CROP,
+                ResizeMode.LETTERBOX_REFLECT_EDGES,
+            },
         )
+        if inference_config.post_processing.type != "nms":
+            raise CorruptedModelPackageError(
+                message="Expected NMS to be the post-processing",
+                help_url="https://todo",
+            )
         trt_config = parse_trt_config(
             config_path=model_package_content["trt_config.json"]
         )
@@ -96,10 +117,28 @@ class YOLOv8ForInstanceSegmentationTRT(
                 engine_host_code_allowed=engine_host_code_allowed,
             )
             execution_context = engine.create_execution_context()
+        inputs, outputs = get_engine_inputs_and_outputs(engine=engine)
+        if len(inputs) != 1:
+            raise CorruptedModelPackageError(
+                message=f"Implementation assume single model input, found: {len(inputs)}.",
+                help_url="https://todo",
+            )
+        if len(outputs) != 2:
+            raise CorruptedModelPackageError(
+                message=f"Implementation assume 2 model outputs, found: {len(outputs)}.",
+                help_url="https://todo",
+            )
+        if "output0" not in outputs or "output1" not in outputs:
+            raise CorruptedModelPackageError(
+                message=f"Expected model outputs to be named `output0` and `output1`, but found: {outputs}.",
+                help_url="https://todo",
+            )
         return cls(
             engine=engine,
+            input_name=inputs[0],
+            output_names=["output0", "output1"],
             class_names=class_names,
-            pre_processing_config=pre_processing_config,
+            inference_config=inference_config,
             trt_config=trt_config,
             device=device,
             execution_context=execution_context,
@@ -109,16 +148,20 @@ class YOLOv8ForInstanceSegmentationTRT(
     def __init__(
         self,
         engine: trt.ICudaEngine,
+        input_name: str,
+        output_names: List[str],
         class_names: List[str],
-        pre_processing_config: PreProcessingConfig,
+        inference_config: InferenceConfig,
         trt_config: TRTConfig,
         device: torch.device,
         cuda_context: cuda.Context,
         execution_context: trt.IExecutionContext,
     ):
         self._engine = engine
+        self._input_name = input_name
+        self._output_names = output_names
         self._class_names = class_names
-        self._pre_processing_config = pre_processing_config
+        self._inference_config = inference_config
         self._trt_config = trt_config
         self._device = device
         self._cuda_context = cuda_context
@@ -137,8 +180,8 @@ class YOLOv8ForInstanceSegmentationTRT(
     ) -> Tuple[torch.Tensor, List[PreProcessingMetadata]]:
         return pre_process_network_input(
             images=images,
-            pre_processing_config=self._pre_processing_config,
-            expected_network_color_format="rgb",
+            image_pre_processing=self._inference_config.image_pre_processing,
+            network_input=self._inference_config.network_input,
             target_device=self._device,
             input_color_format=input_color_format,
         )
@@ -154,8 +197,8 @@ class YOLOv8ForInstanceSegmentationTRT(
                     engine=self._engine,
                     context=self._execution_context,
                     device=self._device,
-                    input_name="images",
-                    outputs=["output0", "output1"],
+                    input_name=self._input_name,
+                    outputs=self._output_names,
                 )
                 return instances, protos
 
@@ -170,13 +213,18 @@ class YOLOv8ForInstanceSegmentationTRT(
         **kwargs,
     ) -> List[InstanceDetections]:
         instances, protos = model_results
-        nms_results = run_nms_for_instance_segmentation(
-            output=instances,
-            conf_thresh=conf_thresh,
-            iou_thresh=iou_thresh,
-            max_detections=max_detections,
-            class_agnostic=class_agnostic,
-        )
+        if self._inference_config.post_processing.fused:
+            nms_results = post_process_nms_fused_model_output(
+                output=instances, conf_thresh=conf_thresh
+            )
+        else:
+            nms_results = run_nms_for_instance_segmentation(
+                output=instances,
+                conf_thresh=conf_thresh,
+                iou_thresh=iou_thresh,
+                max_detections=max_detections,
+                class_agnostic=class_agnostic,
+            )
         final_results = []
         for image_bboxes, image_protos, image_meta in zip(
             nms_results, protos, pre_processing_meta
@@ -201,7 +249,9 @@ class YOLOv8ForInstanceSegmentationTRT(
                 scale_height=image_meta.scale_height,
                 scale_width=image_meta.scale_width,
                 original_size=image_meta.original_size,
+                size_after_pre_processing=image_meta.size_after_pre_processing,
                 inference_size=image_meta.inference_size,
+                static_crop_offset=image_meta.static_crop_offset,
             )
             final_results.append(
                 InstanceDetections(
