@@ -6,21 +6,26 @@ import torch
 from inference_exp import Detections, KeyPoints, KeyPointsDetectionModel
 from inference_exp.configuration import DEFAULT_DEVICE
 from inference_exp.entities import ColorFormat
-from inference_exp.errors import EnvironmentConfigurationError, MissingDependencyError
+from inference_exp.errors import (
+    CorruptedModelPackageError,
+    EnvironmentConfigurationError,
+    MissingDependencyError,
+)
 from inference_exp.models.common.model_packages import get_model_package_contents
 from inference_exp.models.common.onnx import (
-    run_session_via_iobinding,
     run_session_with_batch_size_limit,
     set_execution_provider_defaults,
 )
 from inference_exp.models.common.roboflow.model_packages import (
-    PreProcessingConfig,
+    InferenceConfig,
     PreProcessingMetadata,
+    ResizeMode,
     parse_class_names_file,
+    parse_inference_config,
     parse_key_points_metadata,
-    parse_pre_processing_config,
 )
 from inference_exp.models.common.roboflow.post_processing import (
+    post_process_nms_fused_model_output,
     rescale_key_points_detections,
     run_nms_for_key_points_detection,
 )
@@ -79,7 +84,7 @@ class YOLOv8ForKeyPointsDetectionOnnx(
             model_package_dir=model_name_or_path,
             elements=[
                 "class_names.txt",
-                "environment.json",
+                "inference_config.json",
                 "weights.onnx",
                 "keypoints_metadata.json",
             ],
@@ -87,10 +92,21 @@ class YOLOv8ForKeyPointsDetectionOnnx(
         class_names = parse_class_names_file(
             class_names_path=model_package_content["class_names.txt"]
         )
-        pre_processing_config = parse_pre_processing_config(
-            config_path=model_package_content["environment.json"],
+        inference_config = parse_inference_config(
+            config_path=model_package_content["inference_config.json"],
+            allowed_resize_modes={
+                ResizeMode.STRETCH_TO,
+                ResizeMode.LETTERBOX,
+                ResizeMode.CENTER_CROP,
+                ResizeMode.LETTERBOX_REFLECT_EDGES,
+            },
         )
-        parsed_key_points_metadata = parse_key_points_metadata(
+        if inference_config.post_processing.type != "nms":
+            raise CorruptedModelPackageError(
+                message="Expected NMS to be the post-processing",
+                help_url="https://todo",
+            )
+        parsed_key_points_metadata, skeletons = parse_key_points_metadata(
             key_points_metadata_path=model_package_content["keypoints_metadata.json"]
         )
         session = onnxruntime.InferenceSession(
@@ -100,27 +116,34 @@ class YOLOv8ForKeyPointsDetectionOnnx(
         input_batch_size = session.get_inputs()[0].shape[0]
         if isinstance(input_batch_size, str):
             input_batch_size = None
+        input_name = session.get_inputs()[0].name
         return cls(
             session=session,
+            input_name=input_name,
             class_names=class_names,
-            pre_processing_config=pre_processing_config,
+            inference_config=inference_config,
             device=device,
             input_batch_size=input_batch_size,
             parsed_key_points_metadata=parsed_key_points_metadata,
+            skeletons=skeletons,
         )
 
     def __init__(
         self,
         session: onnxruntime.InferenceSession,
-        pre_processing_config: PreProcessingConfig,
+        input_name: str,
+        inference_config: InferenceConfig,
         class_names: List[str],
         device: torch.device,
         input_batch_size: Optional[int],
         parsed_key_points_metadata: List[List[str]],
+        skeletons: List[List[Tuple[int, int]]],
     ):
         self._session = session
-        self._pre_processing_config = pre_processing_config
+        self._input_name = input_name
+        self._inference_config = inference_config
         self._class_names = class_names
+        self._skeletons = skeletons
         self._device = device
         self._input_batch_size = input_batch_size
         self._session_thread_lock = Lock()
@@ -140,25 +163,31 @@ class YOLOv8ForKeyPointsDetectionOnnx(
     def key_points_classes(self) -> List[List[str]]:
         return self._parsed_key_points_metadata
 
+    @property
+    def skeletons(self) -> List[List[Tuple[int, int]]]:
+        return self._skeletons
+
     def pre_process(
         self,
         images: Union[torch.Tensor, List[torch.Tensor], np.ndarray, List[np.ndarray]],
         input_color_format: Optional[ColorFormat] = None,
+        image_size: Optional[Tuple[int, int]] = None,
         **kwargs,
     ) -> Tuple[torch.Tensor, List[PreProcessingMetadata]]:
         return pre_process_network_input(
             images=images,
-            pre_processing_config=self._pre_processing_config,
-            expected_network_color_format="rgb",
+            image_pre_processing=self._inference_config.image_pre_processing,
+            network_input=self._inference_config.network_input,
             target_device=self._device,
             input_color_format=input_color_format,
+            image_size_wh=image_size,
         )
 
     def forward(self, pre_processed_images: torch.Tensor, **kwargs) -> torch.Tensor:
         with self._session_thread_lock:
             return run_session_with_batch_size_limit(
                 session=self._session,
-                inputs={"images": pre_processed_images},
+                inputs={self._input_name: pre_processed_images},
                 min_batch_size=self._input_batch_size,
                 max_batch_size=self._input_batch_size,
             )[0]
@@ -174,15 +203,20 @@ class YOLOv8ForKeyPointsDetectionOnnx(
         key_points_threshold: float = 0.3,
         **kwargs,
     ) -> Tuple[List[KeyPoints], Optional[List[Detections]]]:
-        nms_results = run_nms_for_key_points_detection(
-            output=model_results,
-            num_classes=len(self._class_names),
-            key_points_slots_in_prediction=self._key_points_slots_in_prediction,
-            conf_thresh=conf_thresh,
-            iou_thresh=iou_thresh,
-            max_detections=max_detections,
-            class_agnostic=class_agnostic,
-        )
+        if self._inference_config.post_processing.fused:
+            nms_results = post_process_nms_fused_model_output(
+                output=model_results, conf_thresh=conf_thresh
+            )
+        else:
+            nms_results = run_nms_for_key_points_detection(
+                output=model_results,
+                num_classes=len(self._class_names),
+                key_points_slots_in_prediction=self._key_points_slots_in_prediction,
+                conf_thresh=conf_thresh,
+                iou_thresh=iou_thresh,
+                max_detections=max_detections,
+                class_agnostic=class_agnostic,
+            )
         rescaled_results = rescale_key_points_detections(
             detections=nms_results,
             images_metadata=pre_processing_meta,
@@ -199,11 +233,15 @@ class YOLOv8ForKeyPointsDetectionOnnx(
                     confidence=result[:, 4],
                 )
             )
-            key_points_reshaped = result[:, 6:].view(result.shape[0], -1, 3)
+            key_points_reshaped = result[:, 6:].view(
+                result.shape[0], self._key_points_slots_in_prediction, 3
+            )
             xy = key_points_reshaped[:, :, :2]
             confidence = key_points_reshaped[:, :, 2]
             key_points_classes_for_instance_class = (
-                self._key_points_classes_for_instances[class_id]
+                (self._key_points_classes_for_instances[class_id])
+                .unsqueeze(1)
+                .to(device=result.device)
             )
             instances_class_mask = (
                 torch.arange(self._key_points_slots_in_prediction, device=result.device)
@@ -211,6 +249,7 @@ class YOLOv8ForKeyPointsDetectionOnnx(
                 .repeat(result.shape[0], 1)
                 < key_points_classes_for_instance_class
             )
+
             confidence_mask = confidence < key_points_threshold
             mask = instances_class_mask & confidence_mask
             xy[mask] = 0.0
