@@ -6,20 +6,25 @@ import torch
 from inference_exp import Detections, ObjectDetectionModel
 from inference_exp.configuration import DEFAULT_DEVICE
 from inference_exp.entities import ColorFormat
-from inference_exp.errors import EnvironmentConfigurationError, MissingDependencyError
+from inference_exp.errors import (
+    CorruptedModelPackageError,
+    EnvironmentConfigurationError,
+    MissingDependencyError,
+)
 from inference_exp.models.common.model_packages import get_model_package_contents
 from inference_exp.models.common.onnx import (
-    run_session_via_iobinding,
     run_session_with_batch_size_limit,
     set_execution_provider_defaults,
 )
 from inference_exp.models.common.roboflow.model_packages import (
-    PreProcessingConfig,
+    InferenceConfig,
     PreProcessingMetadata,
+    ResizeMode,
     parse_class_names_file,
-    parse_pre_processing_config,
+    parse_inference_config,
 )
 from inference_exp.models.common.roboflow.post_processing import (
+    post_process_nms_fused_model_output,
     rescale_detections,
     run_nms_for_object_detection,
 )
@@ -78,16 +83,27 @@ class YOLOv8ForObjectDetectionOnnx(
             model_package_dir=model_name_or_path,
             elements=[
                 "class_names.txt",
-                "environment.json",
+                "inference_config.json",
                 "weights.onnx",
             ],
         )
         class_names = parse_class_names_file(
             class_names_path=model_package_content["class_names.txt"]
         )
-        pre_processing_config = parse_pre_processing_config(
-            config_path=model_package_content["environment.json"],
+        inference_config = parse_inference_config(
+            config_path=model_package_content["inference_config.json"],
+            allowed_resize_modes={
+                ResizeMode.STRETCH_TO,
+                ResizeMode.LETTERBOX,
+                ResizeMode.CENTER_CROP,
+                ResizeMode.LETTERBOX_REFLECT_EDGES,
+            },
         )
+        if inference_config.post_processing.type != "nms":
+            raise CorruptedModelPackageError(
+                message="Expected NMS to be the post-processing",
+                help_url="https://todo",
+            )
         session = onnxruntime.InferenceSession(
             path_or_bytes=model_package_content["weights.onnx"],
             providers=onnx_execution_providers,
@@ -95,10 +111,12 @@ class YOLOv8ForObjectDetectionOnnx(
         input_batch_size = session.get_inputs()[0].shape[0]
         if isinstance(input_batch_size, str):
             input_batch_size = None
+        input_name = session.get_inputs()[0].name
         return cls(
             session=session,
+            input_name=input_name,
             class_names=class_names,
-            pre_processing_config=pre_processing_config,
+            inference_config=inference_config,
             device=device,
             input_batch_size=input_batch_size,
         )
@@ -106,16 +124,23 @@ class YOLOv8ForObjectDetectionOnnx(
     def __init__(
         self,
         session: onnxruntime.InferenceSession,
-        pre_processing_config: PreProcessingConfig,
+        input_name: str,
+        inference_config: InferenceConfig,
         class_names: List[str],
         device: torch.device,
         input_batch_size: Optional[int],
     ):
         self._session = session
-        self._pre_processing_config = pre_processing_config
+        self._input_name = input_name
+        self._inference_config = inference_config
         self._class_names = class_names
         self._device = device
-        self._input_batch_size = input_batch_size
+        self._min_batch_size = input_batch_size
+        self._max_batch_size = (
+            input_batch_size
+            if input_batch_size is not None
+            else inference_config.forward_pass.max_dynamic_batch_size
+        )
         self._session_thread_lock = Lock()
 
     @property
@@ -126,23 +151,25 @@ class YOLOv8ForObjectDetectionOnnx(
         self,
         images: Union[torch.Tensor, List[torch.Tensor], np.ndarray, List[np.ndarray]],
         input_color_format: Optional[ColorFormat] = None,
+        image_size: Optional[Union[Tuple[int, int], int]] = None,
         **kwargs,
     ) -> Tuple[torch.Tensor, List[PreProcessingMetadata]]:
         return pre_process_network_input(
             images=images,
-            pre_processing_config=self._pre_processing_config,
-            expected_network_color_format="rgb",
+            image_pre_processing=self._inference_config.image_pre_processing,
+            network_input=self._inference_config.network_input,
             target_device=self._device,
             input_color_format=input_color_format,
+            image_size_wh=image_size,
         )
 
     def forward(self, pre_processed_images: torch.Tensor, **kwargs) -> torch.Tensor:
         with self._session_thread_lock:
             return run_session_with_batch_size_limit(
                 session=self._session,
-                inputs={"images": pre_processed_images},
-                min_batch_size=self._input_batch_size,
-                max_batch_size=self._input_batch_size,
+                inputs={self._input_name: pre_processed_images},
+                min_batch_size=self._min_batch_size,
+                max_batch_size=self._max_batch_size,
             )[0]
 
     def post_process(
@@ -155,13 +182,18 @@ class YOLOv8ForObjectDetectionOnnx(
         class_agnostic: bool = False,
         **kwargs,
     ) -> List[Detections]:
-        nms_results = run_nms_for_object_detection(
-            output=model_results,
-            conf_thresh=conf_thresh,
-            iou_thresh=iou_thresh,
-            max_detections=max_detections,
-            class_agnostic=class_agnostic,
-        )
+        if self._inference_config.post_processing.fused:
+            nms_results = post_process_nms_fused_model_output(
+                output=model_results, conf_thresh=conf_thresh
+            )
+        else:
+            nms_results = run_nms_for_object_detection(
+                output=model_results,
+                conf_thresh=conf_thresh,
+                iou_thresh=iou_thresh,
+                max_detections=max_detections,
+                class_agnostic=class_agnostic,
+            )
         rescaled_results = rescale_detections(
             detections=nms_results,
             images_metadata=pre_processing_meta,
